==> v0.13.11.tar.gz: git-next/crates/cli/src/repo/tests/handlers/receive_ci_status.rs <==
use std::time::Duration;

//
use super::*;

#[test_log::test(actix::test)]
async fn when_pass_should_advance_main_to_next() -> TestResult {
    //given
    let fs = given::a_filesystem();
    let (open_repository, repo_details) = given::an_open_repository(&fs);
    let next_commit = given::a_named_commit("next");

    //when
    let (addr, log) = when::start_actor_with_open_repository(
        Box::new(open_repository),
        repo_details,
        given::a_forge(),
    );
    addr.send(crate::repo::messages::ReceiveCIStatus::new((
        next_commit.clone(),
        git::forge::commit::Status::Pass,
    )))
    .await?;
    System::current().stop();

    //then
    tracing::debug!(?log, "");
    log.read().map_err(|e| e.to_string()).map(|l| {
        let expected = format!("send: AdvanceMain({next_commit:?})");
        tracing::debug!(%expected,"");
        assert!(l.iter().any(|message| message.contains(&expected)));
    })?;
    Ok(())
}

#[test_log::test(actix::test)]
async fn when_pending_should_recheck_ci_status() -> TestResult {
    //given
    let fs = given::a_filesystem();
    let (open_repository, repo_details) = given::an_open_repository(&fs);
    let next_commit = given::a_named_commit("next");

    //when
    let (addr, log) = when::start_actor_with_open_repository(
        Box::new(open_repository),
        repo_details,
        given::a_forge(),
    );
    addr.send(crate::repo::messages::ReceiveCIStatus::new((
        next_commit.clone(),
        git::forge::commit::Status::Pending,
    )))
    .await?;
    actix_rt::time::sleep(Duration::from_millis(9)).await;
    System::current().stop();

    //then
    tracing::debug!(?log, "");
    log.require_message_containing("send: ValidateRepo")?;
    Ok(())
}

#[test_log::test(actix::test)]
async fn when_fail_should_recheck_after_delay() -> TestResult {
    //given
    let fs = given::a_filesystem();
    let (open_repository, repo_details) = given::an_open_repository(&fs);
    let next_commit = given::a_named_commit("next");

    //when
    let (addr, log) = when::start_actor_with_open_repository(
        Box::new(open_repository),
        repo_details,
        given::a_forge(),
    );
    addr.send(crate::repo::messages::ReceiveCIStatus::new((
        next_commit.clone(),
        git::forge::commit::Status::Fail,
    )))
    .await?;
    actix_rt::time::sleep(Duration::from_millis(9)).await;
    System::current().stop();

    //then
    log.require_message_containing("send: ValidateRepo")?;
    Ok(())
}

#[test_log::test(actix::test)]
async fn when_fail_should_notify_user() -> TestResult {
    //given
    let fs = given::a_filesystem();
    let (open_repository, repo_details) = given::an_open_repository(&fs);
    let next_commit = given::a_named_commit("next");

    //when
    let (addr, log) = when::start_actor_with_open_repository(
        Box::new(open_repository),
        repo_details,
        given::a_forge(),
    );
    addr.send(crate::repo::messages::ReceiveCIStatus::new((
        next_commit.clone(),
        git::forge::commit::Status::Fail,
    )))
    .await?;
    System::current().stop();

    //then
    log.require_message_containing("send: NotifyUser")?;
    Ok(())
}

==> v0.14.1.tar.gz: git-next/crates/cli/src/repo/tests/handlers/receive_ci_status.rs <==
use std::time::Duration;

use git::forge::commit::Status;

use crate::{repo::messages::ReceiveCIStatus, tell};

//
use super::*;

#[test_log::test(tokio::test)]
async fn when_pass_should_advance_main_to_next() -> TestResult {
    //given
    let fs = given::a_filesystem();
    let (open_repository, repo_details) = given::an_open_repository(&fs);
    let next_commit = given::a_named_commit("next");

    //when
    let (addr, log) = when::start_actor_with_open_repository(
        Box::new(open_repository),
        repo_details,
        given::a_forge(),
    );
    tell!(
        addr,
        ReceiveCIStatus::new((next_commit.clone(), Status::Pass))
    )?;

    //then
    tracing::debug!(?log, "");
    log.require_message_containing(format!("send: AdvanceMain({next_commit:?})"))
        .await?;
    Ok(())
}

#[test_log::test(tokio::test)]
async fn when_pending_should_recheck_ci_status() -> TestResult {
    //given
    let fs = given::a_filesystem();
    let (open_repository, repo_details) = given::an_open_repository(&fs);
    let next_commit = given::a_named_commit("next");

    //when
    let (addr, log) = when::start_actor_with_open_repository(
        Box::new(open_repository),
        repo_details,
        given::a_forge(),
    );
    tell!(
        addr,
        ReceiveCIStatus::new((next_commit.clone(), Status::Pending))
    )?;
    tokio::time::sleep(Duration::from_millis(9)).await;

    //then
    tracing::debug!(?log, "");
    log.require_message_containing("send: ValidateRepo").await?;
    Ok(())
}

#[test_log::test(tokio::test)]
async fn when_fail_should_recheck_after_delay() -> TestResult {
    //given
    let fs = given::a_filesystem();
    let (open_repository, repo_details) = given::an_open_repository(&fs);
    let next_commit = given::a_named_commit("next");
    let (repo_actor_ref, log) = when::start_actor_with_open_repository(
        Box::new(open_repository),
        repo_details,
        given::a_forge(),
    );

    //when
    tell!(
        repo_actor_ref,
        ReceiveCIStatus::new((next_commit.clone(), Status::Fail))
    )?;

    //then
    tokio::time::sleep(Duration::from_millis(100)).await; // otherwise this test can be flakey
    log.require_message_containing("send: ValidateRepo").await?;
    Ok(())
}

#[test_log::test(tokio::test)]
async fn when_fail_should_notify_user() -> TestResult {
    //given
    let fs = given::a_filesystem();
    let (open_repository, repo_details) = given::an_open_repository(&fs);
    let next_commit = given::a_named_commit("next");

    //when
    let (addr, log) = when::start_actor_with_open_repository(
        Box::new(open_repository),
        repo_details,
        given::a_forge(),
    );
    tell!(
        addr,
        ReceiveCIStatus::new((next_commit.clone(), Status::Fail))
    )?;

    //then
    log.require_message_containing("send: NotifyUser").await?;
    Ok(())
}
